//=====================================
//	Author:		L. Spiro
//	Date:		February 22nd, 2005 (incidentally, my birthday [now 23; I got a card from my mother]).
//	Copyright:	None!  I really dont care what you do with this
//				project!  Send it to your friends, pretend you wrote it,
//				I dont care (but you would have to be one sad sad person
//				to do that)!
//	Purpose:	To demonstrate the specifications of Memory Hacking Softwares
//				breakpoint function imports, as exported from the DLL files.
//
//	These files can be modified in any way by anyone for any reason.
//	These files can be used in commercial products free of royalty and without
//	mentioning my name (L. Spiro).
//	These files can and should be used as a template for creating your own
//	breakpoint-handling routines for use with Memory Hacking Software.
//
//	This project was compiled with and for Microsoft Visual C++ .NET 7.1.3088.
//	If you have an environment lower than this, you will have to create
//	a new project and manually add the files to it!
//
//	All functions are optional except the function(s) used to handle breakpoints.
//	Without them, this DLL has no purpose.
//=====================================

#include "stdafx.h"
#include "SampleBreakpointHandler.h"

// DllMain is a very bad place to put initialization code because the DLL will be loaded and unloaded
//	repeatedly before being officially loaded.
//	When it is officially loaded, StartUp() will be called one time, and only one time.
//	When it is officially unloaded, CleanUp() will be called once and only once.
BOOL APIENTRY DllMain( HANDLE hModule,
					  DWORD  ul_reason_for_call,
					  LPVOID lpReserved ) {
	switch ( ul_reason_for_call ) {
		case DLL_PROCESS_ATTACH : {
			// Best left blank.
			break;
		}
		case DLL_THREAD_ATTACH : {
			break;
		}
		case DLL_THREAD_DETACH : {
			break;
		}
		case DLL_PROCESS_DETACH : {
			// Best left blank.
			break;
		}
	}
    return TRUE;
}


//=====================================
//	Initialization.
//=====================================

// StartUp() is called one time during the execution of Memory Hacking Software.
//	This is your chance to perform all initialization routines.
// The time it is loaded depends on how Memory Hacking Software finds it.
//	When you first select a function from this DLL to be used for handling breakpoints,
//		it will be called once, at that time, and then it will not be called again until
//		the next time you run Memory Hacking Software, even if you import more functions
//		from this DLL.
//		Note that this is the very first time you select a function from this DLL.  After
//		you have selected ant function from this DLLit once, it is added to a list so that
//		you can select it again at later dates more easily.
//	If any functions from this DLL are loaded into the above-mentioned list, then StartUp()
//		is called once when you successfully attach the debugger to a process, rather than
//		when you load functions from this DLL.
//
//	This function receives the handle of the process being debugged and its process ID.
// Be sure to use __stdcall and remember to add it to the .DEF file!  This function MUST be
//	exported via the .DEF file!
//
// This function is optional.
void __stdcall StartUp( HANDLE hProcess, DWORD dwProcessID ) {
	MessageBox( NULL, "Initialize data.", "StartUp", MB_OK );
}

// CleanUp() is called one time during the execution of Memory Hacking Software.
//	This is your chance to free any memory allocated and/or close any handles you have
//	created.  Do not attempt to close the hProcess handle.
// This function will only be called when Memory Hacking Software is closing, and will
//	only be called one time, no matter how many functions have been imported from it.
//
//	This function receives the handle of the process being debugged and its process ID.
// Be sure to use __stdcall and remember to add it to the .DEF file!  This function MUST be
//	exported via the .DEF file!
//
// This function is optional.
void __stdcall CleanUp( HANDLE hProcess, DWORD dwProcessID ) {
	MessageBox( NULL, "Free data.", "CleanUp", MB_OK );
}

// ProcessAttach() is called every time the debugger attaches to a process.
//  This is called immediately after StartUp().
// The purpose of this function is to give a process-specific initialization routine.
// StartUp() is called one time during the entire period Memory Hacking Software is open,
//	even if the process being debugged closes and opens again, and even if other processes
//	are opened for debugging.
// This function is called every time the debugger attaches to a process for debugging.
//
//	This function receives the handle of the process being debugged and its process ID.
// Be sure to use __stdcall and remember to add it to the .DEF file!  This function MUST be
//	exported via the .DEF file!
//
// This function is optional.
void __stdcall ProcessAttach( HANDLE hProcess, DWORD dwProcessID ) {
	MessageBox( NULL, "Attaching the debugger to the process.", "ProcessAttach", MB_OK );
}

// ProcessDetach() is called when the current process being debugged closes and the debugger stops
//	debugging.
// This is specifically for freeing resources allocated in ProcessAttach().
// There is one ProcessDetach() call for each ProcessAttach(), and both may be called many times
//	inbetween calls to StartUp() and CleanUp().
//
//	This function receives the handle of the process being debugged and its process ID.
// Be sure to use __stdcall and remember to add it to the .DEF file!  This function MUST be
//	exported via the .DEF file!
//
// This function is optional.
void __stdcall ProcessDetach( HANDLE hProcess, DWORD dwProcessID ) {
	MessageBox( NULL, "Detaching from the process (the process is closing).", "ProcessDetach", MB_OK );
}


//=====================================
//	Breakpoint handlers.
//=====================================

// This is an example of a breakpoint function.
// After being assigned to a breakpoint, it will be called when the breakpoint is hit.
// Only one breakpoint can have a given address, and this is how you can tell which breakpoint
//	is being hit (if needed).
//
// From here, you can perform any operations you like.
// Here are a few functions that may be of interest:
//	GetThreadContext()
//	SetThreadContext()
//	ReadProcessMemory()
//	WriteProcessMemory()
// Be sure to use __stdcall.
//	This function can be exported by any means you desire; Memory Hacking Software will automatically
//	detects its name, however using __declspec( dllexport ) may make it a bit hard to read.
EXPORT_API void __stdcall BPFFunction( LPVOID bpAddress, const PROCESS_INFORMATION &piProcessInfo ) {
	char szBuffer[256];
	sprintf( szBuffer, "Breakpoint hit at 0x%.8X.", bpAddress );
	MessageBox( NULL, szBuffer, "Breakpoint Hit", MB_OK );
	return;
}